  #include "addressbook.h"
    #include "ui_addressbook.h"

    AddressBook::AddressBook(QWidget *parent)
        : QWidget(parent), ui(new Ui::AddressBook)
    {
        ui->setupUi(this);

        //! [extract objects]
        nameEdit = new QLineEdit;
        nameEdit = ui->nameEdit;
        nameEdit->setReadOnly(true);

        addressText = new QTextEdit;
        addressText = ui->addressText;
        addressText->setReadOnly(true);

        addButton = new QPushButton;
        addButton = ui->addButton;

        submitButton = new QPushButton;
        submitButton = ui->submitButton;
        submitButton->hide();

        cancelButton = new QPushButton;
        cancelButton = ui->cancelButton;
        cancelButton->hide();
        //! [extract objects]

        //! [signal slot]
        connect(addButton, SIGNAL(clicked()), this,
                    SLOT(addContact()));
        connect(submitButton, SIGNAL(clicked()), this,
                    SLOT(submitContact()));
        connect(cancelButton, SIGNAL(clicked()), this,
                    SLOT(cancel()));
        //! [signal slot]

        //! [window title]
        setWindowTitle(tr("Simple Address Book"));
        //! [window title]
    }

    AddressBook::~AddressBook()
    {
        delete ui;
    }

    void AddressBook::addContact()
    {
        oldName = nameEdit->text();
        oldAddress = addressText->toPlainText();

        nameEdit->clear();
        addressText->clear();

        nameEdit->setReadOnly(false);
        nameEdit->setFocus(Qt::OtherFocusReason);
        addressText->setReadOnly(false);

        addButton->setEnabled(false);
        submitButton->show();
        cancelButton->show();
    }

    void AddressBook::submitContact()
    {
        QString name = nameEdit->text();
        QString address = addressText->toPlainText();

        if (name == "" || address == "") {
            QMessageBox::information(this, tr("Empty Field"),
                tr("Please enter a name and address."));
            return;
        }

        if (!contacts.contains(name)) {
            contacts.insert(name, address);
            QMessageBox::information(this, tr("Add Successful"),
                tr("\"%1\" has been added to your address book.").arg(name));
            return;
        } else {
            QMessageBox::information(this, tr("Add Unsuccessful"),
                tr("Sorry, \"%1\" is already in your address book.").arg(name));
            return;
        }

        if (contacts.isEmpty()) {
            nameEdit->clear();
            addressText->clear();
        }

        nameEdit->setReadOnly(true);
        addressText->setReadOnly(true);
        addButton->setEnabled(true);
        submitButton->hide();
        cancelButton->hide();
    }

    void AddressBook::cancel()
    {
        nameEdit->setText(oldName);
        nameEdit->setReadOnly(true);

        addressText->setText(oldAddress);
        addressText->setReadOnly(true);

        addButton->setEnabled(true);
        submitButton->hide();
        cancelButton->hide();
    }